package com.mh.user.mapper;

import com.mh.user.entity.AlarmInfoEntity;
import com.mh.user.entity.AlarmValueSetEntity;
import com.mh.user.mapper.provider.AlarmInfoProvider;
import org.apache.ibatis.annotations.*;
import org.springframework.security.core.parameters.P;

import java.util.List;

@Mapper
public interface AlarmInfoMapper {

    /**
     * 报警管理
     * 保存报警信息
     * @param alarmInfoEntity
     */
    @Insert("insert into alarm_info(building_id,building_name,alarm_time,alarm_device,alarm_type,operate) values (" +
            "#{buildingId},#{buildingName},#{alarmTime},#{alarmDevice},#{alarmType},#{operate})")
    void saveAlarmInfo(AlarmInfoEntity alarmInfoEntity);

    /**
     * 报警管理
     * 根据日期修改报警信息
     * @param alarmInfoEntity
     */
    @Update("<script>" +
            "  update Alarm_Info set  " +
            "  <if test='buildingId!=null'>  building_id = #{buildingId} </if>" +
            "  <if test='buildingName!=null'> , building_name = #{buildingName} </if>" +
            "  <if test='alarmTime!=null'> , alarm_time = #{alarmTime} </if>" +
            "  <if test='alarmDevice!=null'> , alarm_device = #{alarmDevice} </if>" +
            "  <if test='alarmType!=null'> , alarm_type = #{alarmType} </if>" +
            "  <if test='operate!=null'> , operate = #{operate} </if>" +
            "   where id = #{id} " +
            "</script>")
    void updateAlarmInfo(AlarmInfoEntity alarmInfoEntity);

    /**
     * 报警管理
     * 根据日期、设备、楼栋查询报警信息
     * @param alarmTime
     * @param buildingId
     * @param page
     * @param limit
     * @return
     */
    @SelectProvider(type = AlarmInfoProvider.class,method = "queryAlarmInfo")
    @Results({
            @Result(column = "building_id",property = "buildingId" ),
            @Result(column = "building_name", property = "buildingName"),
            @Result(column = "alarm_time", property = "alarmTime"),
            @Result(column = "device_type", property = "deviceType"),
            @Result(column = "alarm_type", property = "alarmType"),
            @Result(column = "deal_state", property = "dealState"),
            @Result(column = "id", property = "id"),
            @Result(column = "device_addr", property = "deviceAddr")
    })
    List<AlarmInfoEntity> queryAlarmInfo(@Param("alarmTime") String alarmTime,
                                         @Param("alarmType") String alarmType,
                                         @Param("buildingId") String buildingId,
                                         @Param("dealState") String dealState,
                                         @Param("page") int page,
                                         @Param("limit") int limit);

    /**
     * 报警管理
     * 根据日期、设备类型查询记录数
     * @return
     */
    @SelectProvider(type = AlarmInfoProvider.class,method = "getAlarmInfoCount")
    int getAlarmInfoCount(@Param("alarmTime") String alarmTime,
                          @Param("alarmType") String alarmType,
                          @Param("buildingId") String buildingId,
                          @Param("dealState") String dealState,
                          @Param("page") int page,
                          @Param("limit") int limit);
    /**
     * 报警管理
     * 根据报警编号删除报警信息
     * @param id
     */
    @Delete("delete from Alarm_Info where id=#{id}")
    void deleteAlarmInfo(@Param("id") String id);
    ///////////////////////////////////////
    //警报设置
    @Insert("insert into alarm_value_set(building_id,startTime,endTime,startTime2,endTime2,startTime3,endTime3,lowValue,lowValue2,lowValue3,itemType) values (" +
            "#{buildingId},#{startTime},#{endTime},#{startTime2},#{endTime2},#{startTime3},#{endTime3},#{lowValue},#{lowValue2},#{lowValue3},#{itemType})")
    void saveAlarmValueSet(AlarmValueSetEntity alarmValueSetEntity);

    //修改
    @Update("<script>" +
            "  update alarm_value_set set  " +
            "  <if test='buildingId!=null'>  building_id = #{buildingId} </if>" +
            "  <if test='startTime!=null'> , startTime = #{startTime} </if>" +
            "  <if test='endTime!=null'> , endTime = #{endTime} </if>" +
            "  <if test='startTime2!=null'> , startTime2 = #{startTime2} </if>" +
            "  <if test='endTime2!=null'> , endTime2 = #{endTime2} </if>" +
            "  <if test='startTime3!=null'> , startTime3 = #{startTime3} </if>" +
            "  <if test='endTime3!=null'> , endTime3 = #{endTime3} </if>" +
            "  <if test='lowValue!=null'> , lowValue = #{lowValue} </if>" +
            "  <if test='lowValue2!=null'> , lowValue2 = #{lowValue2} </if>" +
            "  <if test='lowValue3!=null'> , lowValue3 = #{lowValue3} </if>" +
            "  <if test='itemType!=null'> , itemType = #{itemType} </if>" +
            "   where id = #{id} " +
            "</script>")
    void updateAlarmValueSet(AlarmValueSetEntity alarmValueSetEntity);

    //查询报警值设置记录
    @SelectProvider(type = AlarmInfoProvider.class,method = "queryAlarmValueSet")
    @Results({
            @Result(column = "building_id",property = "buildingId" ),
            @Result(column = "building_name",property = "buildingName" ),
            @Result(column = "startTime", property = "startTime"),
            @Result(column = "endTime", property = "endTime"),
            @Result(column = "startTime2", property = "startTime2"),
            @Result(column = "endTime2", property = "endTime2"),
            @Result(column = "startTime3", property = "startTime3"),
            @Result(column = "endTime3", property = "endTime3"),
            @Result(column = "lowValue", property = "lowValue"),
            @Result(column = "lowValue2", property = "lowValue2"),
            @Result(column = "lowValue3", property = "lowValue3"),
            @Result(column = "itemType", property = "itemType"),
            @Result(column = "id", property = "id")
    })
    List<AlarmValueSetEntity> queryAlarmValueSet(@Param("buildingId") String buildingId,@Param("itemType") String itemType,@Param("page") int page,@Param("limit") int limit);

    //查询报警值设置记录数
    @SelectProvider(type = AlarmInfoProvider.class,method = "getAlarmValueSetCount")
    int getAlarmValueSetCount(@Param("buildingId") String buildingId,@Param("itemType") String itemType);

    //删除设置记录
    @Delete("delete from alarm_value_set where  id=#{id}")
    void delAlarmValueSet(@Param("id") String id);

    //修改状态
    @Update("update alarm_info set deal_state=#{dealState} where id=#{id}")
    void updateDealState(@Param("id") String id,@Param("dealState") String dealState);

    //按楼栋编号删除设置记录
    @Delete("delete from alarm_value_set where  building_id=#{buildingId} and itemType=#{itemType}")
    void delAlarmValueSet2(@Param("buildingId") String buildingId,@Param("itemType") String itemType);

}
