package com.mh.user.mapper;

import com.mh.user.entity.DeviceFloorEntity;
import com.mh.user.entity.DeviceFloorTempEntity;
import com.mh.user.entity.DeviceInstallTempEntity;
import com.mh.user.mapper.provider.DeviceFloorProvider;
import com.mh.user.mapper.provider.DeviceInstallProvider;
import com.mh.user.model.DeviceModel;
import com.mh.user.model.PumpModel;
import org.apache.ibatis.annotations.*;
import org.apache.ibatis.mapping.StatementType;

import java.util.Date;
import java.util.List;

@Mapper
public interface DeviceFloorMapper {

    /**
     * 楼面设备模块
     * 保存设备信息
     * @param deviceFloorEntity
     */
    @Insert("insert into device_floor(device_name,device_type,brand,model,specs,building_id,installer,install_date,is_use) values (" +
            " #{deviceName},#{deviceType},#{brand},#{model},#{specs},#{buildingId},#{installer},getDate(),#{use})")
    int saveDevice(DeviceFloorEntity deviceFloorEntity);

    /**
     * 楼面设备模块
     * 保存设备信息
     * @param deviceFloorEntity
     */
    @Update("<script>" +
            "  update device_floor set install_date=getDate() " +
            "  <if test='deviceName!=null'>, device_name = #{deviceName} </if>" +
            "  <if test='deviceType!=null'> , device_type = #{deviceType} </if>" +
            "  <if test='brand!=null'> , brand = #{brand} </if>" +
            "  <if test='model!=null'> , model = #{model} </if>" +
            "  <if test='specs!=null'> , specs = #{specs} </if>" +
            "  <if test='installer!=null'> , installer = #{installer} </if>" +
            "  <if test='buildingId!=null'> , building_id = #{buildingId} </if>" +
            "  <if test='use!=null'> , is_use = #{use} </if>" +
            "   where id = #{id} " +
            "</script>")
    int updateDevice(DeviceFloorEntity deviceFloorEntity);

    /**
     * 楼面设备模块
     * 查询设备资料
     * @param buildingId
     * @param deviceType
     * @param page
     * @param limit
     * @return
     */
    @Results(value = {
            @Result(property="id",column="id"),
            @Result(property="deviceName",column="device_name"),
            @Result(property="deviceType",column="device_type"),
            @Result(property ="buildingId",column ="building_id"),
            @Result(property ="buildingName",column ="building_name"),
            @Result(property="installDate",column="install_date"),
            @Result(property="model",column="model"),
            @Result(property="specs",column="specs"),
            @Result(property="use",column="is_use")
    })
    @SelectProvider(type = DeviceFloorProvider.class,method = "queryDeviceFloor")
    List<DeviceFloorEntity> queryDeviceFloor(@Param("buildingId") String buildingId,@Param("deviceType") String deviceType,@Param("page") int page, @Param("limit") int limit);

    /**
     * 楼面设备模块
     * 获取设备查询的总条数
     * @param buildingId
     * @param deviceType
     * @param page
     * @param limit
     * @return
     */
    @SelectProvider(type = DeviceFloorProvider.class,method = "getCount")
    int getCount(@Param("buildingId") String buildingId,@Param("deviceType") String deviceType,@Param("page") int page, @Param("limit") int limit);

    //根据楼栋、类型查询设备名称
    @Results(value = {
            @Result(property="deviceName",column="device_name")
    })
    @Select("select * from device_floor  where building_id=#{buildingId}  and  device_type=#{deviceType}")
    List<DeviceModel> selectDeviceName(@Param("buildingId") String buildingId, @Param("deviceType") String deviceType);

    //根据楼栋、类型、设备名称判断是否存在
    @Select("select count(*) from device_floor  where building_id=#{buildingId}  and  device_type=#{deviceType}  and  device_name=#{deviceName}")
    int selectDeviceCount(@Param("buildingId") String buildingId,@Param("deviceType") String deviceType, @Param("deviceName") String deviceName);

    @Results(value = {
            @Result(property="id",column="id"),
            @Result(property="deviceName",column="device_name")
    })
    @Select("select * from device_floor  where building_id=#{buildingId}  and  device_type=#{deviceType} and  pump_id=#{pumpId}")
    DeviceModel selectTankName(@Param("buildingId") String buildingId, @Param("deviceType") String deviceType,@Param("pumpId") String pumpId);

    //根据设备名称、类型查询设备编号
    @Select("select id from device_floor where  device_name=#{deviceName} and  device_type=#{deviceType}  and  building_id=#{buildingId}")
    String selectDeviceId(@Param("deviceName") String deviceName,@Param("deviceType") String deviceType,@Param("buildingId") String buildingId);

    //根据设备类型查询设备编号
    @Results(value = {
            @Result(property="pumpId",column="id"),
            @Result(property="pumpName",column="device_name")
    })
    @Select("select id,device_name from device_floor where  device_type=#{deviceType}  and  building_id=#{buildingId}")
    PumpModel selectDeviceId2(@Param("deviceType") String deviceType, @Param("buildingId") String buildingId);
    /**
     * 楼面设备模块
     * 根据编号删除设备信息
     * @param id
     */
    @Delete("delete from device_floor where id=#{id}")
    int deleteDevice(String id);

    //EXCEL导入相关处理函数
    // 清临时表
    @Delete("DELETE FROM device_floor_temp")
    void deleteDevice_floor_temp();

    // 保存到临时表
    @Insert("insert into device_floor_temp(device_name,device_type,building_id,install_date,model,specs,row_id)" +
            "                   values(#{deviceName},#{deviceType},#{buildingId},getDate(),#{model},#{specs},#{rowId})")
    void insertDevice_floor_temp(  @Param("deviceName")    String deviceName,
                                   @Param("deviceType")   String deviceType,
                                   @Param("buildingId")   String buildingId,
                                   @Param("model")     String model,
                                   @Param("specs")     String specs,
                                   @Param("rowId")     String rowId
    );

    // EXCEL导入查询EXCEL内容
    @Select("SELECT device_name,device_type,building_id,install_date, " +
            "       model,specs,row_id,  " +
            "      (CASE " +
            "        when remarks is not null then remarks + ',行号为'  + convert(varchar(100),Row_id) " +
            "        else '' " +
            "        end" +
            "       ) as remarks " +
            "   FROM device_floor_temp " +
            "   ORDER BY ISNULL(remarks,'')　DESC ,row_id  ")
    @Options(statementType = StatementType.CALLABLE)
    @Results({
            @Result(column = "device_name", property = "deviceName"),
            @Result(column = "device_type", property = "deviceType"),
            @Result(column = "model", property = "model"),
            @Result(column = "specs", property = "specs"),
            @Result(column = "building_id", property = "buildingId"),
            @Result(column = "install_date", property = "installDate"),
            @Result(column = "remarks", property = "remarks"),
            @Result(column = "row_id", property = "rowId")
    })
    List<DeviceFloorTempEntity> queryExcelDevices();

    //查询Excel导入的数据的记录数
    @Select("SELECT COUNT(*) FROM device_floor_temp ")
    int queryExcelDevicesCount();

    /**
     * @author nxr
     * @title ：判断导入资料数据的合法
     * @description ：在导入中的数据有重复
     * @updateTime 2022-06-19
     * @throws ：
     */
    @Update("update  device_floor_temp  " +
            "  set Remarks = '数据中表号有重复'  " +
            "  where Remarks is null  " +
            "  and  device_id in ( select  ISNULL(device_floor_temp.device_id,'')  " +
            "                            from  device_floor_temp  " +
            "                          group by device_id " +
            "                          having count(*) > 1)")
    void updateDevice_floor_temp__multiple();

}
