package com.mh.user.serialport;

import com.mh.user.entity.DeviceCodeParamEntity;
import com.mh.user.service.DeviceCodeParamService;
import com.mh.user.utils.AnalysisReceiveOrder485;
import com.mh.user.utils.SendOrderUtils;
import com.mh.user.utils.SpringBeanUtil;
import gnu.io.SerialPort;
import gnu.io.SerialPortEvent;
import gnu.io.SerialPortEventListener;
import lombok.SneakyThrows;
import org.springframework.context.ApplicationContext;
import java.nio.ByteBuffer;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 串口监听器
 * @author mar
 * @date 2021年08月10日 09:56
 */
public class SerialPortListener implements SerialPortEventListener {

    private  String receiveStr = null;
//    public static SerialPort serialPort = null;
    public  SerialPort serialPort = null;
    private int num = 0;
    private int size = 0;
    List<DeviceCodeParamEntity> deviceManageEntityList;

    // 调用service
    ApplicationContext context = SpringBeanUtil.getApplicationContext();
    DeviceCodeParamService deviceCodeParamService = context.getBean(DeviceCodeParamService.class);
    AnalysisReceiveOrder485 analysisReceiveOrder485 = new AnalysisReceiveOrder485();

    @SneakyThrows
    @Override
    public void serialEvent(SerialPortEvent arg0) {
        if (arg0.getEventType() == SerialPortEvent.DATA_AVAILABLE) {
            SerialPortUtil serialPortUtil = SerialPortUtil.getSerialPortUtil();
//            byte[] bytes = serialPortUtil.readFromPort(serialPort);
            byte[] bytes = serialPortUtil.readFromPort(SerialPortThread.serialPort);
            String byteStr = new String(bytes, 0, bytes.length).trim();
            System.out.println("===========start===========");
            receiveStr = receiveStr + printHexString(bytes);
            receiveStr = receiveStr.replace("null", "");
            receiveStr = receiveStr.replace(" ", "");
            System.out.println(new Date() + "【字节数组转16进制字符串】：" + receiveStr);
            System.out.println("接受服务器数据：" + receiveStr + "，大小： " + receiveStr.length());
            System.out.println("===========end===========");
            size = deviceManageEntityList.size();
        }
    }

    public void serialPortSend(String comName){

        //查看所有串口
        SerialPortUtil serialPortUtil = SerialPortUtil.getSerialPortUtil();
        ArrayList<String> port = serialPortUtil.findPort();
        System.out.println("发现全部串口：" + port);
        if (port.contains(comName)){
            //打开该对应comName名字的串口
            if (serialPort != null) { //打开之前先关闭
                serialPortUtil.closePort(serialPort);
                serialPort = null;
            }
            System.out.println("打开指定串口:" + comName);
            serialPort = serialPortUtil.openPort(comName, 2400, SerialPort.DATABITS_8, SerialPort.PARITY_EVEN, SerialPort.PARITY_ODD);
            //给对应的serialPort添加监听器
            serialPortUtil.addListener(serialPort, new SerialPortListener());
            try{
                //生成对应的采集指令
                deviceManageEntityList = deviceCodeParamService.queryCodeParam2(comName);
                System.out.println("打印列表："+deviceManageEntityList);
                size = deviceManageEntityList.size();
//                SendOrderUtils.sendSerialPort(deviceManageEntityList.get(0), serialPort);
                Thread.sleep(1000);
            }catch (Exception e){
                e.printStackTrace();
            }
        }else {
            System.out.println("串口："+comName+"不存在!" );
        }
    }


    /**
     * 字节数组转16进制字符串
     * @param b 字节数组
     * @return 16进制字符串
     */
    public static String printHexString(byte[] b) {
        StringBuilder sbf = new StringBuilder();
        for (byte value : b) {
            String hex = Integer.toHexString(value & 0xFF);
            if (hex.length() == 1) {
                hex = '0' + hex;
            }
            sbf.append(hex.toUpperCase()).append(" ");
        }
        return sbf.toString().trim();
    }

    /**
     * 十六进制字符串转byte[]
     * @param hex 十六进制字符串
     * @return byte[]
     */
    public static byte[] hexStr2Byte(String hex) {
        if (hex == null) {
            return new byte[] {};
        }

        // 奇数位补0
        if (hex.length() % 2 != 0) {
            hex = "0" + hex;
        }

        int length = hex.length();
        ByteBuffer buffer = ByteBuffer.allocate(length / 2);
        for (int i = 0; i < length; i++) {
            String hexStr = hex.charAt(i) + "";
            i++;
            hexStr += hex.charAt(i);
            byte b = (byte) Integer.parseInt(hexStr, 16);
            buffer.put(b);
        }
        return buffer.array();
    }

    /**
     * 16进制转换成为string类型字符串
     * @param s 待转换字符串
     */
    public static String hexStringToString(String s) {
        if (s == null || "".equals(s)) {
            return null;
        }
        s = s.replace(" ", "");
        byte[] baKeyword = new byte[s.length() / 2];
        for (int i = 0; i < baKeyword.length; i++) {
            try {
                baKeyword[i] = (byte) (0xff & Integer.parseInt(s.substring(i * 2, i * 2 + 2), 16));
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        try {
            s = new String(baKeyword, StandardCharsets.UTF_8);
        } catch (Exception e1) {
            e1.printStackTrace();
        }
        return s;
    }

}



