package com.mh.user.serialport;

import com.mh.common.utils.StringUtils;
import com.mh.user.constants.Constant;
import com.mh.user.entity.DeviceCodeParamEntity;
import com.mh.user.service.BuildingService;
import com.mh.user.service.DeviceInstallService;
import com.mh.user.service.NowDataService;
import com.mh.user.utils.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.ApplicationContext;

import java.io.IOException;

import purejavacomm.SerialPort;
import purejavacomm.SerialPortEvent;
import purejavacomm.SerialPortEventListener;

/**
 * @author nxr
 * @title ：
 * @description ： 串口发送和接收处理,操作类
 * @updateTime 2022-08-10
 * @throws ：
 */
@Slf4j
public class SerialPortSingle2 {

    // 调用service
    ApplicationContext context = SpringBeanUtil.getApplicationContext();
    AnalysisReceiveOrder485 analysisReceiveOrder485 = new AnalysisReceiveOrder485();
    DeviceInstallService deviceInstallService = context.getBean(DeviceInstallService.class);
    NowDataService nowDataService = context.getBean(NowDataService.class);
    BuildingService buildingService = context.getBean(BuildingService.class);

    public String serialPortSend(DeviceCodeParamEntity deviceCodeParamEntity) {
        SerialPort serialPort = null;
        String rtData = "fail";
        String comName = deviceCodeParamEntity.getDataCom().toUpperCase();
        try {
            try {
                int baudrate = deviceCodeParamEntity.getBaudrate();
                String parity = deviceCodeParamEntity.getParity();
                if (parity == null || parity.equals("") || parity.equalsIgnoreCase("none")) {
                    serialPort = SerialTool.openPort(comName, baudrate, SerialPort.DATABITS_8, SerialPort.STOPBITS_1, SerialPort.PARITY_NONE);
                } else {
                    serialPort = SerialTool.openPort(comName, baudrate, SerialPort.DATABITS_8, SerialPort.STOPBITS_1, SerialPort.PARITY_EVEN);
                }
                if (serialPort != null) {
                    //向串口发送指令
                    log.info("-----------------------------单抄向串口" + serialPort.getName() + "发送指令！-----------------------------");
                    String sendStr = SendOrderUtils.getSendStr(deviceCodeParamEntity);
                    SerialTool.sendToPort(SerialTool.HexString2Bytes(sendStr), serialPort, sendStr, deviceCodeParamEntity.getDeviceType());
                    CacheTools.initReceiveMsg(serialPort.getName());
                    SerialPort finalSerialPort = serialPort;
                    SerialTool.addListener(serialPortEvent -> {
                        try {
                            for (int i = 0; i < 5; i++) {
                                Thread.sleep(1000);
                                byte[] bytes = SerialTool.readFromPort(finalSerialPort);
                                if (bytes!= null && bytes.length > 0) {
                                    if (Constant.WEB_FLAG) {
                                        CacheTools.setReceiveMsg(finalSerialPort.getName(), ExchangeStringUtil.printHexString(bytes));
                                        break;
                                    }
                                }
                            }
                        } catch (Exception e) {
                            log.error("单抄串口" + finalSerialPort + "异常，关闭串口", e);
                        }
                    }, serialPort);
                }
            } catch (Exception e) {
                SerialTool.closePort(serialPort);
                log.info("单抄串口" + serialPort + "异常,没有数据返回！关闭串口", e);
                return "fail";
            }
            String receiveStr;
            if (serialPort !=  null) {
                receiveStr = CacheTools.waitReceiveMsg(serialPort.getName());
                //去掉空格和null
                if (StringUtils.isBlank(receiveStr)) {
                    return "fail";
                }
                receiveStr = receiveStr.replace("null", "").replace(" ", "");
                log.info("串口" + serialPort.getName() + "接收数据：" + receiveStr + "，大小： " + receiveStr.length());
                //返回值全部变成大写
                String receiveData = receiveStr.toUpperCase();
                //截取去掉FE
                String deviceType = deviceCodeParamEntity.getDeviceType();
                String deviceAddr = deviceCodeParamEntity.getDeviceAddr();
                String dataStr = "";
                if (receiveData.length() > 8 && ("水表".equals(deviceType) || "电表".equals(deviceType))) {
                    String str1 = receiveData.substring(0, 8);
                    String str2 = receiveData.substring(8);
                    dataStr = str1.replace("FE", "") + str2;
                } else {
                    dataStr = receiveData;
                }
                String registerAddr = deviceCodeParamEntity.getRegisterAddr();
                String brand = deviceCodeParamEntity.getBrand();
                String buildingId = deviceCodeParamEntity.getBuildingId();
                String buildingName = buildingService.queryBuildingName(buildingId); //查询楼栋名称

                deviceInstallService.updateOnline(deviceAddr, deviceType, buildingId, "在线"); //设备在线
                log.info(deviceType + "在线,设备号：" + deviceAddr + ",所属楼栋：" + buildingName);
                if (deviceType.equals("热泵")) {
                    String strState = nowDataService.selectState(buildingId, deviceAddr);
                    if (strState != null && strState.equals("离线")) { //采集到数据
                        nowDataService.updateRunState(buildingId, deviceAddr, "不运行", buildingName); //监控界面状态表热泵在线状态
                    }
                }
                try {
                    if ((dataStr.length() == 18 || dataStr.length() == 70 || dataStr.length() == 72 || dataStr.length() == 44) && deviceType.equals("水表")) {
                        rtData = analysisReceiveOrder485.analysisWtMeterOrder4852(dataStr, registerAddr, brand, buildingId, buildingName);
                    } else if ((dataStr.length() == 36 || dataStr.length() == 44 || dataStr.length() == 40 || dataStr.length() == 50) && deviceType.equals("电表")) {
                        rtData = analysisReceiveOrder485.analysisMeterOrder4852(dataStr, registerAddr, brand, buildingId, buildingName);
                    } else if (deviceType.equals("压变")) {
                        rtData = analysisReceiveOrder485.analysisPressureOrder4852(dataStr, registerAddr, brand, buildingId, buildingName);
                    } else if ((dataStr.length() == 30) && deviceType.equals("状态检测")) {//五路状态读取,兼容旧版系统
                        analysisReceiveOrder485.analysisStateOrder485(dataStr, registerAddr, brand, buildingId, buildingName);
                    } else if (deviceType.equals("水位开关") && (StringUtils.isBlank(registerAddr) || (registerAddr.equals("0018") || registerAddr.equals("0017")))) {
                        rtData = analysisReceiveOrder485.analysisRelayOrder4852(dataStr, registerAddr, brand, buildingId, buildingName);
                        log.info("水位开关读取或者设置返回值==>{}", rtData);
                    } else if (deviceType.equals("热泵")) {
                        rtData = analysisReceiveOrder485.analysisPumpOrder4852(dataStr, registerAddr, brand, buildingId, buildingName);
                    } else if (deviceType.equals("时控")) {
                        rtData = analysisReceiveOrder485.analysisTimeSetOrder4852(dataStr, registerAddr, brand, buildingId, buildingName);
                        log.info("时控读取或者设置返回值==>{}", rtData);
                    } else if (deviceType.equals("水位开关") && !StringUtils.isBlank(registerAddr) &&  registerAddr.equals("0010")) { //热泵状态
                        rtData = analysisReceiveOrder485.analysisPumpStateOrder2(dataStr, registerAddr, brand, buildingId, buildingName);
                    } else if (deviceType.equals("温度变送器")) {
                        rtData = analysisReceiveOrder485.analysisMulTempOrder4852(dataStr, registerAddr, brand, buildingId, buildingName);
                    } else if (deviceType.equals("热泵状态")) {
                        rtData = analysisReceiveOrder485.analysisPumpStateOrder2(dataStr, registerAddr, brand, buildingId, buildingName);
                    }
                } catch (Exception e) {
                    log.error(deviceCodeParamEntity.getDeviceType() + "单抄保存数据库失败！", e);
                    SerialTool.closePort(serialPort);
                    return "fail";
                }
            } else {
                log.info("串口为空，无法进行采集控制");
                return "fail";
            }
            SerialTool.closePort(serialPort);
            Thread.sleep(200);
            log.info("-----------------------------" + serialPort.getName() + "单抄结束！-----------------------------");
            return rtData;
        } catch (Exception e) {
            assert serialPort != null;
            log.info("单抄串口" + serialPort.getName() + "异常,没有数据返回！关闭串口", e);
        } finally {
            try {
                if (null != serialPort) {
                    SerialTool.closePort(serialPort);
                }
            } catch (IOException e) {
                log.error("关闭串口异常！", e);
            }
        }
        return "fail";
    }
}
