package com.mh.user.service.impl;

import com.mh.user.entity.DataResultEntity;
import com.mh.user.entity.DeviceInstallEntity;
import com.mh.user.mapper.DataResultMapper;
import com.mh.user.mapper.DeviceInstallMapper;
import com.mh.user.service.DataResultService;
import com.mh.user.service.DeviceInstallService;
import com.mh.user.utils.ExchangeStringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

@Service
public class DataResultServiceImpl implements DataResultService {

    @Autowired
    DataResultMapper dataResultMapper;

    @Autowired
    DeviceInstallMapper deviceInstallMapper;

    @Autowired
    DeviceInstallService deviceInstallService;

    @Override
    public void saveDataResult(DataResultEntity dataResultEntity) {
        SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        double lastValue=0;
        double calcValue=0;
        double initValue=0;
        double curValue=0;
        double dayValue=0;
        int days=0;
        try{
            Date date=new Date(); //获取系统日期
            if (dataResultEntity.getCurDate()==null){
                dataResultEntity.setCurDate(date);
            }
            //从安装表获取设备信息
            DeviceInstallEntity deviceInstallEntity=deviceInstallMapper.selectDevice(dataResultEntity.getDeviceAddr(),dataResultEntity.getDeviceType(),dataResultEntity.getBuildingId());
            double ratio=deviceInstallEntity.getRatio(); //倍率
            initValue=deviceInstallEntity.getInitValue();
            dayValue=deviceInstallEntity.getDayValue();
            if(dataResultEntity.getDeviceType().equals("电表")){
                dayValue=1000;
            }else{
                dayValue=100;
            }
            //判断是否有记录
            int r = dataResultMapper.selectDataResultCount(sdf1.format(dataResultEntity.getCurDate()),
                    dataResultEntity.getDeviceAddr(),dataResultEntity.getDeviceType());
            if (r==0){//插入记录
                DataResultEntity data=new DataResultEntity();
                data.setDeviceAddr(dataResultEntity.getDeviceAddr());       //通讯地址
                data.setDeviceType(dataResultEntity.getDeviceType());       //设备类型
                String curDate=sdf1.format(dataResultEntity.getCurDate());
                data.setCurDate(sdf1.parse(curDate));                       //当前日期
                data.setCurValue(dataResultEntity.getCurValue());           //当前读数
                curValue=dataResultEntity.getCurValue();
                data.setBuildingId(deviceInstallEntity.getBuildingId());    //楼栋编号
                data.setRatio(ratio);                                       //倍率
                lastValue=deviceInstallEntity.getLastValue();               //上次读数
                calcValue=(dataResultEntity.getCurValue()-lastValue)*ratio; //计算用量
                DecimalFormat df = new DecimalFormat("0.000");
                calcValue=Double.parseDouble(df.format(calcValue));

                data.setCalcValue(calcValue);                               //用量
                if (deviceInstallEntity.getLastDate()!=null){               //上次抄表日期
                    String lastDate=sdf1.format(deviceInstallEntity.getLastDate());
                    data.setLastDate(sdf1.parse(lastDate));
                }else{
                    String lastDate=sdf1.format(date);
                    data.setLastDate(sdf1.parse(lastDate));
                }
                data.setLastValue(lastValue);

                //判断读数,并保存数据
                days=(int)ExchangeStringUtil.daysBetween(dataResultEntity.getCurDate(),deviceInstallEntity.getLastDate()); //计算相差天数
                if (calcValue>=0 && calcValue<=dayValue){
                    dataResultMapper.saveDataResult(data);         //插入新的记录
                }else if(calcValue>dayValue){
                   if(days>0){
                       if(calcValue/days<=dayValue){
                           dataResultMapper.saveDataResult(data);
                       }
                   }
                }

            }else {//修改记录的curValue、calcValue
                DataResultEntity data2=dataResultMapper.selectDataResult(sdf1.format(dataResultEntity.getCurDate()),dataResultEntity.getDeviceAddr(),dataResultEntity.getDeviceType());
                lastValue=data2.getLastValue();       //安装基表上次读数
                calcValue=dataResultEntity.getCurValue()-lastValue; //计算用量
                String curDate=sdf1.format(dataResultEntity.getCurDate());
                data2.setCurDate(sdf1.parse(curDate));//当前日期
                data2.setCurValue(dataResultEntity.getCurValue());//当前读数
                data2.setCalcValue(calcValue);//用量

                //判断读数,并保存数据
                days=(int)ExchangeStringUtil.daysBetween(dataResultEntity.getCurDate(),data2.getLastDate()); //计算相差天数
                if (calcValue>=0 && calcValue<=dayValue){
                    dataResultMapper.updateDataResult(data2);  //更新记录
                }else if(calcValue>dayValue){
                    if(days>0){
                        if(calcValue/days<=dayValue){
                            dataResultMapper.saveDataResult(data2);
                        }
                    }
                }
            }

            //修改安装表中lastValue,lastDate,ini_value的值
            if (calcValue>=0 && calcValue<=dayValue){
                deviceInstallMapper.updateLastValue(deviceInstallEntity.getId(),String.valueOf(dataResultEntity.getCurValue()),date);
            }else if(calcValue>dayValue){
                if(days>0){
                    if(calcValue/days<=dayValue){
                        deviceInstallMapper.updateLastValue(deviceInstallEntity.getId(),String.valueOf(dataResultEntity.getCurValue()),date);
                    }
                }
            }
            if(initValue==0){//第一次采集的时候
                deviceInstallMapper.updateLastValue(deviceInstallEntity.getId(),String.valueOf(dataResultEntity.getCurValue()),date);
                deviceInstallMapper.updateInitValue(dataResultEntity.getDeviceAddr(),dataResultEntity.getDeviceType(),dataResultEntity.getBuildingId(),String.valueOf(curValue));
            }
        }catch (Exception e){
            //e.printStackTrace();
        }
    }

    @Override
    public void updateDataResult(DataResultEntity dataResultEntity) {
        dataResultMapper.updateDataResult(dataResultEntity);
    }

    @Override
    public List<DataResultEntity> queryDataResult(String buildingId, String startDate, String endDate,String deviceType, int page, int limit) {
        return dataResultMapper.queryDataResult(buildingId,startDate,endDate,deviceType,page,limit);
    }

    @Override
    public int getDataResultCount(String buildingId, String startDate, String endDate,String deviceType, int page, int limit) {
        return dataResultMapper.getDataResultCount(buildingId,startDate,endDate,deviceType,page,limit);
    }

}
