package com.mh.user.service.impl;

import com.mh.user.entity.DeviceCodeParamEntity;
import com.mh.user.entity.DeviceInstallEntity;
import com.mh.user.entity.DeviceInstallTempEntity;
import com.mh.user.entity.UploadDeviceInstallEntity;
import com.mh.user.mapper.DeviceCodeParamMapper;
import com.mh.user.mapper.DeviceInstallMapper;
import com.mh.user.model.DeviceModel;
import com.mh.user.service.BuildingService;
import com.mh.user.service.DeviceInstallService;
import com.mh.user.utils.CacheUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Random;

@Slf4j
@Service
public class DeviceInstallServiceImpl implements DeviceInstallService {

    @Autowired
    private DeviceInstallMapper deviceInstallMapper;

    @Autowired
    private BuildingService buildingService;

    @Autowired
    private DeviceCodeParamMapper deviceCodeParamMapper;

    @Override
    public int saveDevice(DeviceInstallEntity deviceInstallEntity) {
        String buildingName = buildingService.selectBuildingNameById(deviceInstallEntity.getBuildingId());
        deviceInstallEntity.setBuildingName(buildingName);
        this.createParamCode(deviceInstallEntity);
        return deviceInstallMapper.saveDevice(deviceInstallEntity);
    }

    /**
     * 生成采集信息内容
     * @param deviceInstallEntity
     */
    @Override
    public void createParamCode(DeviceInstallEntity deviceInstallEntity) {
        DeviceCodeParamEntity deviceCodeParamEntity = new DeviceCodeParamEntity();
        BeanUtils.copyProperties(deviceInstallEntity,deviceCodeParamEntity);
        deviceCodeParamEntity.setBaudrate(deviceInstallEntity.getBaudRate());
        deviceCodeParamEntity.setThread(deviceInstallEntity.getDataCom().toLowerCase().replace("com",""));
        // 顺便生成采集信息
        switch (deviceInstallEntity.getDeviceType()) {
            case "压变":
            case "温度变送器":
            case "温控":
                deviceCodeParamMapper.insertDeviceCodeParamList(Collections.singletonList(deviceCodeParamEntity));
                break;
            case "电表":
            case "水表":
            case "状态检测":
            case "热泵状态":
                deviceCodeParamMapper.insertDeviceCodeParamList2(Collections.singletonList(deviceCodeParamEntity));
                break;
            case "水位开关":
                switch (deviceInstallEntity.getBrand()) {
                    case "远向":
                        deviceCodeParamEntity.setRegisterAddr("0018");//读水位设定
                        deviceCodeParamEntity.setFunCode("03");
                        deviceCodeParamMapper.insertDeviceCodeParamList3(Collections.singletonList(deviceCodeParamEntity));

                        deviceCodeParamEntity.setRegisterAddr("0010");//读热泵状态、上水用水状态
                        deviceCodeParamEntity.setFunCode("03");
                        break;
                    case "中凯":
                        deviceCodeParamEntity.setRegisterAddr("0017");//读水位设定
                        deviceCodeParamEntity.setFunCode("03");
                        break;
                    default:
                        deviceCodeParamEntity.setRegisterAddr("0102");//读水位设定
                        deviceCodeParamEntity.setFunCode("0102");
                        break;
                }
                deviceCodeParamMapper.insertDeviceCodeParamList3(Collections.singletonList(deviceCodeParamEntity));
                break;
            case "热泵":
                // 需要区分型号，然后在设置值到表里面
                switch (deviceInstallEntity.getBrand()) {
                    case "美的":
                        // 插入device_code_param表
                        deviceCodeParamEntity.setRegisterAddr("0007"); //实际水温
                        deviceCodeParamEntity.setFunCode("03");
                        deviceCodeParamMapper.insertDeviceCodeParamList(Collections.singletonList(deviceCodeParamEntity));

                        // 插入device_code_param1表
                        deviceCodeParamEntity.setRegisterAddr("0641"); //运行状态
                        deviceCodeParamEntity.setFunCode("03");
                        deviceCodeParamMapper.insertDeviceCodeParamList2(Collections.singletonList(deviceCodeParamEntity));

                        // 插入device_code_param2表
                        deviceCodeParamEntity.setRegisterAddr("0642"); //设定温度
                        deviceCodeParamEntity.setFunCode("03");
                        deviceCodeParamMapper.insertDeviceCodeParamList(Collections.singletonList(deviceCodeParamEntity));

                        // 插入device_code_param2表
                        deviceCodeParamEntity.setRegisterAddr("000B"); //故障状态
                        deviceCodeParamEntity.setFunCode("03");
                        deviceCodeParamMapper.insertDeviceCodeParamList(Collections.singletonList(deviceCodeParamEntity));
                    break;
                    case "美的2":
                        // 插入device_code_param表
                        deviceCodeParamEntity.setRegisterAddr("0064"); //实际水温
                        deviceCodeParamEntity.setFunCode("03");
                        deviceCodeParamMapper.insertDeviceCodeParamList(Collections.singletonList(deviceCodeParamEntity));

                        // 插入device_code_param1表
                        deviceCodeParamEntity.setRegisterAddr("0BBD"); //运行状态
                        deviceCodeParamEntity.setFunCode("03");
                        deviceCodeParamMapper.insertDeviceCodeParamList2(Collections.singletonList(deviceCodeParamEntity));

                        // 插入device_code_param2表
                        deviceCodeParamEntity.setRegisterAddr("0003"); //设定温度
                        deviceCodeParamEntity.setFunCode("03");
                        deviceCodeParamMapper.insertDeviceCodeParamList(Collections.singletonList(deviceCodeParamEntity));

                        // 插入device_code_param2表
                        deviceCodeParamEntity.setRegisterAddr("0BBB"); //故障状态
                        deviceCodeParamEntity.setFunCode("03");
                        deviceCodeParamMapper.insertDeviceCodeParamList(Collections.singletonList(deviceCodeParamEntity));
                        break;
                    case "瑞星":
                        // 插入device_code_param表
                        deviceCodeParamEntity.setRegisterAddr("0046"); //实际水温
                        deviceCodeParamEntity.setFunCode("03");
                        deviceCodeParamMapper.insertDeviceCodeParamList(Collections.singletonList(deviceCodeParamEntity));

                        // 插入device_code_param1表
                        deviceCodeParamEntity.setRegisterAddr("0001"); //运行状态
                        deviceCodeParamEntity.setFunCode("03");
                        deviceCodeParamMapper.insertDeviceCodeParamList2(Collections.singletonList(deviceCodeParamEntity));

                        // 插入device_code_param2表
                        deviceCodeParamEntity.setRegisterAddr("000A"); //设定温度
                        deviceCodeParamEntity.setFunCode("03");
                        deviceCodeParamMapper.insertDeviceCodeParamList(Collections.singletonList(deviceCodeParamEntity));

                        // 插入device_code_param2表
                        deviceCodeParamEntity.setRegisterAddr("0050"); //故障状态
                        deviceCodeParamEntity.setFunCode("03");
                        deviceCodeParamMapper.insertDeviceCodeParamList(Collections.singletonList(deviceCodeParamEntity));
                        break;
                    default:
                        break;
                }
                break;
            default:
                break;
        }
        // 删除之前的缓存
        CacheUtil.getInstance().deleteCache();
    }

    @Override
    public int updateDevice(DeviceInstallEntity deviceInstallEntity) {
        return deviceInstallMapper.updateDevice(deviceInstallEntity);
    }

    @Override
    public void updateLastValue(Long id, String lastValue, Date lastDate) {
        deviceInstallMapper.updateLastValue(id,lastValue,lastDate);
    }

    @Override
    public List<DeviceInstallEntity> getAllDevice(int page, int limit) {
        return deviceInstallMapper.getAllDevice(page,limit);
    }

    @Override
    public List<DeviceInstallEntity> selectDeviceParams() {
        return deviceInstallMapper.selectDeviceParams();
    }

    @Override
    public List<DeviceInstallEntity> selectDeviceParams2() {
        return deviceInstallMapper.selectDeviceParams2();
    }

    @Override
    public List<DeviceInstallEntity> selectDevicePump() {
        return deviceInstallMapper.selectDevicePump();
    }

    // 查询所有启用的设备
    @Override
    public int getAllCount(){
        return deviceInstallMapper.getAllCount();
    }

    @Override
    public List<DeviceInstallEntity> queryDevice(String buildingId,String deviceType, String startDate, String endDate,String isOnline, String isUse,String isFault,int page, int limit) {
        return deviceInstallMapper.queryDevice(buildingId,deviceType,startDate, endDate,isOnline,isUse, isFault,page, limit);
    }

    @Override
    public int getCount(String buildingId,String deviceType, String startDate, String endDate,String isOnline, String isUse, String isFault, int page, int limit) {
        return deviceInstallMapper.getCount(buildingId,deviceType, startDate, endDate,isOnline,isUse, isFault, page, limit);
    }

    //查询设备故障情况
    @Override
    public int getIsFaultCount(String isFault,String deviceType){
        return deviceInstallMapper.getIsFaultCount(isFault,deviceType);
    }

    //查询设备在线情况
    @Override
    public int getIsOnlineCount(String isOnline,String deviceType){
        return deviceInstallMapper.getIsOnlineCount(isOnline,deviceType);
    }

    @Override
    public void updateOnline(String deviceAddr, String deviceType,String buildingId,String isOnline) {
        deviceInstallMapper.updateOnline(deviceAddr,deviceType,buildingId,isOnline);
    }

    @Override
    public void updateNotOnline(String deviceAddr, String deviceType,String buildingId,String isOnline) {
        deviceInstallMapper.updateNotOnline(deviceAddr,deviceType,buildingId,isOnline);
    }

    //根据通讯地址和设备类型查询对应的设备信息
    @Override
    public DeviceInstallEntity selectDevice(String deviceAddr, String deviceType,String buildingId) {
        return deviceInstallMapper.selectDevice(deviceAddr,deviceType,buildingId);
    }

    @Override
    public int selectDeviceCount(String deviceAddr, String deviceType) {
        return deviceInstallMapper.selectDeviceCount(deviceAddr,deviceType);
    }

    @Override
    public int deleteDevice(List<DeviceInstallEntity> records) {
        for(DeviceInstallEntity record:records) {
            deviceInstallMapper.deleteDevice(record.getId().toString());
        }
        return 0;
    }

    @Override
    public int deleteDevice(String id) {
        // 根据id查询对应的deviceInstall
        DeviceInstallEntity oldEntity = deviceInstallMapper.selectDeviceById(Long.valueOf(id));
        // 删除全部的device_code_param值
        this.deleteParamCode(oldEntity);
        return deviceInstallMapper.deleteDevice(id);
    }

    //Excel导入相关处理函数
    @Override
    public void deleteDevice_install_temp() {
        deviceInstallMapper.deleteDevice_install_temp();
    }

    @Override
    public void insertDevice_install_temp(UploadDeviceInstallEntity uploadDeviceInstallEntity) {
        String deviceAddr = uploadDeviceInstallEntity.getDeviceAddr();
        String deviceName   = uploadDeviceInstallEntity.getDeviceName();
        String deviceType  = uploadDeviceInstallEntity.getDeviceType();
        int baudRate     = uploadDeviceInstallEntity.getBaudRate();
        String dataCom    = uploadDeviceInstallEntity.getDataCom();
        double ratio    = uploadDeviceInstallEntity.getRatio();
        String buildingId    = uploadDeviceInstallEntity.getBuildingId();
        String rowId         = uploadDeviceInstallEntity.getRowId();
        deviceInstallMapper.insertDevice_install_temp(deviceAddr, deviceName, deviceType, baudRate, dataCom, ratio, buildingId, rowId);
    }

    // 查询Excel导入的数据
    @Override
    public List<DeviceInstallTempEntity> queryExcelDevices(){
        return deviceInstallMapper.queryExcelDevices();
    };

    // 查询Excel导入的数据的记录数
    @Override
    public int queryExcelDevicesCount(){
        return deviceInstallMapper.queryExcelDevicesCount();
    };

    // 在导入中的数据有重复
    @Override
    public void updateDevice_install_temp__multiple() {
        deviceInstallMapper.updateDevice_install_temp__multiple();
    }

    /**
     * @author nxr
     * @title ：
     * @description ：判断导入资料数据的合法性
     * @updateTime 2022-06-19
     * @throws ：
     */
    public void updateDevice_install_temp(){
        //判断通讯地址，在导入中的数据有重复
        deviceInstallMapper.updateDevice_install_temp__multiple();
    }

    //从临时表查询插入
    @Override
    public void insertFromDevice_install_temp() {
        deviceInstallMapper.insertFromDevice_install_temp();
        deviceInstallMapper.deleteDevice_install_temp();
    }

    //修改设备启用状态
    @Override
    public void updateDeviceIsUse(String isUse, String deviceAddr) {
        deviceInstallMapper.updateDeviceIsUse(isUse,deviceAddr);
    }

    @Override
    public List<DeviceModel> selectDevices(String buildingId, String deviceType) {
        return deviceInstallMapper.selectDevices(buildingId, deviceType);
    }

    @Override
    public void updateDeviceFault(String isFault, String deviceAddr, String deviceType) {
        deviceInstallMapper.updateDeviceFault(isFault,deviceAddr,deviceType);
    }

    //查询设备品牌
    @Override
    public String selectBrand(String buildingId,String deviceAddr) {
        return deviceInstallMapper.selectBrand(buildingId,deviceAddr);
    }

    @Override
    public void updateIsUse(String isUse, String deviceAddr, String deviceType, String buildingId) {
        deviceInstallMapper.updateIsUse(isUse, deviceAddr, deviceType, buildingId);
    }

    @Override
    public void updateIsUse2(String isUse, String deviceAddr, String deviceType, String buildingId) {
        deviceInstallMapper.updateIsUse2(isUse, deviceAddr, deviceType, buildingId);
    }

    @Override
    public void updateIsUse3(String isUse, String deviceAddr, String deviceType, String buildingId) {
        deviceInstallMapper.updateIsUse3(isUse, deviceAddr, deviceType, buildingId);
    }

    @Override
    public void deletePump(String deviceAddr, String deviceType, String buildingId) {
        deviceInstallMapper.deletePump(deviceAddr, deviceType, buildingId);
    }

    @Override
    public void updateInitValue(String deviceAddr, String deviceType, String buildingId, String initValue) {
        deviceInstallMapper.updateInitValue(deviceAddr, deviceType, buildingId, initValue);
    }

    @Override
    public String selectLastDate(String deviceType, String deviceAddr,String buildingId) {
        return deviceInstallMapper.selectLastDate(deviceType,deviceAddr,buildingId);
    }

    @Override
    public int judgePump(String buildingId) {
        return deviceInstallMapper.judgePump(buildingId);
    }

    @Override
    public String selectSeat(String deviceType, String deviceAddr, String buildingId) {
        return deviceInstallMapper.selectSeat(deviceType, deviceAddr, buildingId);
    }

    @Override
    public boolean updateDeviation(Integer buildingId, Integer deviceType, Integer param, String readValue, String realValue) {
        String deviceTypeStr = "水表";
        if (deviceType == 0) {
            // 更新水表
            deviceTypeStr = "水表";
        } else if (deviceType == 1) {
            // 更新电表
            deviceTypeStr = "电表";
        }
        String dataValue = new BigDecimal(realValue).subtract(new BigDecimal(readValue)).toString();
        try {
            deviceInstallMapper.updateDeviation(buildingId, deviceTypeStr, param, dataValue);
            return true;
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public Double selectDeviceDeviation(String deviceAddr, String deviceType, String buildingId) {
        return deviceInstallMapper.selectDeviceDeviation(deviceAddr, deviceType, buildingId);
    }

    @Override
    public DeviceInstallEntity selectDeviceById(Long id) {
        return deviceInstallMapper.selectDeviceById(id);
    }

    @Override
    public void deleteParamCode(DeviceInstallEntity oldEntity) {
        deviceCodeParamMapper.deleteParamCode1(oldEntity);
        deviceCodeParamMapper.deleteParamCode2(oldEntity);
        deviceCodeParamMapper.deleteParamCode3(oldEntity);
    }

    @Override
    public void updateLastValueByOther(String addr, String strWtLevel, String deviceType, String buildingId) {
        deviceInstallMapper.updateLastValueByOther(addr, strWtLevel, deviceType, buildingId);
    }
}

