package com.mh.user.service.impl;

import com.mh.user.entity.*;
import com.mh.user.mapper.DeviceFloorMapper;
import com.mh.user.mapper.DeviceInstallMapper;
import com.mh.user.mapper.NowDataMapper;
import com.mh.user.mapper.NowPublicDataMapper;
import com.mh.user.model.DeviceModel;
import com.mh.user.model.PumpModel;
import com.mh.user.model.WaterLevelModel;
import com.mh.user.service.BuildingService;
import com.mh.user.service.DeviceInstallService;
import com.mh.user.service.NowDataService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.concurrent.locks.ReentrantLock;

@Slf4j
@Service
public class NowDataServiceImpl implements NowDataService {

    @Autowired
    NowDataMapper nowDataMapper;

    @Autowired
    NowPublicDataMapper nowPublicDataMapper;

    @Autowired
    DeviceInstallService deviceInstallService;

    @Autowired
    DeviceInstallMapper deviceInstallMapper;

    @Autowired
    DeviceFloorMapper deviceFloorMapper;

    @Autowired
    BuildingService buildingService;

    ReentrantLock lockA = new ReentrantLock();

    ReentrantLock lockB = new ReentrantLock();

    @Override
    public void saveNowData(NowDataEntity nowDataEntity) {
        nowDataMapper.saveNowData(nowDataEntity);
    }

    //保存热泵采集数据
    @Override
    public void saveNowHistoryData(String pumpId, String deviceType, String strData,String dataType,String buildingId) {
        SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date date=new Date();
        String strDate=sdf1.format(date);
        strDate=strDate.substring(0,13)+":00:00";

        NowDataEntity data=new NowDataEntity();
        try{
            if (dataType.equals("tempSet")){         //温度设定
                data.setTempSet(strData);
            }else if (dataType.equals("waterTemp")){ //水箱水温
                data.setWaterTemp(strData);
            }else if (dataType.equals("runState")){  //运行状态
                data.setRunState(strData);
            }else if (dataType.equals("isFault")){   //是否故障
                data.setIsFault(strData);
            }
            //从安装表获取设备信息
            DeviceInstallEntity deviceInstallEntity=deviceInstallMapper.selectDevice(pumpId,deviceType,buildingId);
            //获取对应水箱信息
            DeviceModel tank=deviceFloorMapper.selectTankName(buildingId,"水箱",pumpId);
            if (tank!=null){
                data.setTankId(String.valueOf(tank.getId())); //对应水箱编号
                data.setTankName(tank.getDeviceName());       //对应水箱名称
            }
            String buildingName=buildingService.queryBuildingName(buildingId);
            //判断实时表是否有记录
            int r1 = nowDataMapper.selectNowDataCount(deviceInstallEntity.getBuildingId(),pumpId);//判断now_data表是否存在记录
            if(r1==0){
                data.setPumpId(pumpId); //通讯地址
                data.setPumpName(deviceInstallEntity.getDeviceName()); //通讯地址
                data.setBuildingId(deviceInstallEntity.getBuildingId());
                data.setBuildingName(buildingName);
                nowDataMapper.saveNowData(data);
            } else {
                NowDataEntity data1=nowDataMapper.selectNowData(deviceInstallEntity.getBuildingId(),pumpId);
                if (data1!=null){
                    if (dataType.equals("tempSet")){         //温度设定
                        data1.setTempSet(strData);
                    }else if (dataType.equals("waterTemp")){ //水箱水温
                        data1.setWaterTemp(strData);
                    }else if (dataType.equals("runState")){  //运行状态
                        data1.setRunState(strData);
                    }else if (dataType.equals("isFault")){   //是否故障
                        data1.setIsFault(strData);
                    }
                    nowDataMapper.updateNowData(data1);
                }
            }
            //判断历史表是否有记录
            int r2 = nowDataMapper.selectHistoryDataCount(strDate,deviceInstallEntity.getBuildingId(),pumpId);//判断now_data表是否存在记录
            if (r2==0){
                data.setPumpId(pumpId); //通讯地址
                data.setPumpName(deviceInstallEntity.getDeviceName()); //通讯地址
                data.setBuildingId(deviceInstallEntity.getBuildingId());
                data.setBuildingName(buildingName);
                data.setCurDate(strDate);
                nowDataMapper.saveHistoryData(data);
            }else {
                NowDataEntity data2=nowDataMapper.selectHistoryData(strDate,deviceInstallEntity.getBuildingId(),pumpId);
                if (data2!=null){
                    if (dataType.equals("tempSet")){         //温度设定
                        data2.setTempSet(strData);
                    }else if (dataType.equals("waterTemp")){ //水箱水温
                        data2.setWaterTemp(strData);
                    }else if (dataType.equals("runState")){  //运行状态
                        data2.setRunState(strData);
                    }else if (dataType.equals("isFault")){   //是否故障
                        data2.setIsFault(strData);
                    }
                    nowDataMapper.updateHistoryData(data2);
                }
            }
        }catch (Exception e){
            e.printStackTrace();
        }
    }

    //非热泵温控、状态检测数据
    @Override
    public void saveNowHistoryData2(String deviceAddr, String deviceType, String strData, String dataType,String buildingId) {
        NowDataEntity data=new NowDataEntity();
        SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date date=new Date();
        String strDate=sdf1.format(date);
        strDate=strDate.substring(0,13)+":00:00";

        try{
            if (dataType.equals("tempSet")){ //温度设定
                data.setTempSet(strData);
            }else if (dataType.equals("waterTemp")){ //水箱水温
                data.setWaterTemp(strData);
            }else if (dataType.equals("runState")){  //运行状态
                data.setRunState(strData);
            }else if (dataType.equals("isFault")){   //是否故障
                data.setIsFault(strData);
            }else if (dataType.equals("levelSet")){  //水位设定
                data.setLevelSet(strData);
            }else if (dataType.equals("waterLevel")){//实际水位
                data.setWaterLevel(strData);
            }
            //从安装表获取设备信息
            DeviceInstallEntity deviceInstallEntity=deviceInstallMapper.selectDevice(deviceAddr,deviceType,buildingId);
            String buildingName=buildingService.queryBuildingName(buildingId);
            data.setBuildingName(buildingName);
            if (deviceType.equals("温控")) {
                //为了兼容旧系统，旧系统热泵信息保存在楼面表，从楼面表获取热泵资料
                PumpModel pump=deviceFloorMapper.selectDeviceId2("热泵",buildingId);
                String pumpId="1";
                String pumpName="热泵1";
                if (pump!=null){
                    pumpId=pump.getPumpId();
                    pumpName=pump.getPumpName();
                    //获取对应水箱信息
                    DeviceModel tank=deviceFloorMapper.selectTankName(buildingId,"水箱",pumpId);
                    if (tank!=null){
                        data.setTankId(String.valueOf(tank.getId()));      //对应水箱编号
                        data.setTankName(tank.getDeviceName());    //对应水箱名称
                    }else{
                        data.setTankId("1");      //对应水箱编号
                        data.setTankName("水箱1");    //对应水箱名称
                    }
                }
                data.setPumpId(pumpId); //热泵编号
                data.setPumpName(pumpName); //通讯地址
            }
            data.setBuildingId(buildingId);
            data.setCurDate(strDate);
            //判断实时表是否有记录
            int r1 = nowDataMapper.selectNowDataCount2(buildingId);//判断now_data表是否存在记录
            if (r1==0){
                nowDataMapper.saveNowData(data);
            }else {
                if (deviceType.equals("压变")) {
                    String seat=deviceInstallService.selectSeat("压变",deviceAddr,buildingId);
                    log.info("-------楼栋："+buildingName+",类型："+deviceType+",地址："+deviceAddr+",区位："+seat+",数值："+strData+"------");
                    if (seat!=null){
                        nowDataMapper.nowDataWaterLevel(strData,seat,buildingId);
                    }else{
                        nowDataMapper.updateNowData2(data);
                    }
                }else if (deviceType.equals("水位开关")){
                    String seat=deviceInstallService.selectSeat("水位开关",deviceAddr,buildingId);
                    log.info("-------楼栋："+buildingName+",类型："+deviceType+",地址："+deviceAddr+",区位："+seat+",数值："+strData+"------");
                    if (seat!=null){
                        nowDataMapper.nowDataLevelSet(strData,seat,buildingId);
                    }else{
                        nowDataMapper.updateNowData2(data);
                    }
                }else{
                    nowDataMapper.updateNowData2(data);
                }
            }
            //判断历史表是否有记录
            int r2 = nowDataMapper.selectHistoryDataCount2(strDate,buildingId);//判断history_Data表是否存在记录
            if (r2==0){
                nowDataMapper.saveHistoryData(data);
            }else {
                if (deviceType.equals("压变")) {
                    String seat=deviceInstallService.selectSeat("压变",deviceAddr,buildingId);
                    if (seat!=null){
                        nowDataMapper.historyDataWaterLevel(strData,seat,buildingId,"");
                        log.info("------楼栋:"+buildingName+",历史记录压变区位:"+seat+"------");
                    }else{
                        nowDataMapper.updateHistoryData2(data);
                    }
                }else if (deviceType.equals("水位开关")){
                    String seat=deviceInstallService.selectSeat("水位开关",deviceAddr,buildingId);
                    if (seat!=null){
                        nowDataMapper.historyDataLevelSet(strData,seat,buildingId,"");
                        log.info("------楼栋:"+buildingName+",历史记录水位开关区位:"+seat+"------");
                    }else{
                        nowDataMapper.updateHistoryData2(data);
                    }
                }else{
                    nowDataMapper.updateHistoryData2(data);
                }
            }
        }catch (Exception e){
            e.printStackTrace();
        }
    }

    //非热泵温度变送器、状态检测相关数据
    @Override
    public void saveNowHistoryData3(String pumpId, String deviceType, String strData,String dataType,String buildingId) {
        SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date date=new Date();
        String strDate=sdf1.format(date);
        strDate=strDate.substring(0,13)+":00:00";
        NowDataEntity data=new NowDataEntity();
        try{
            if (dataType.equals("tempSet")){         //温度设定
                data.setTempSet(strData);
            }else if (dataType.equals("waterTemp")){ //水箱水温
                data.setWaterTemp(strData);
            }else if (dataType.equals("runState")){  //运行状态
                data.setRunState(strData);
            }else if (dataType.equals("isFault")){   //是否故障
                data.setIsFault(strData);
            }
            String buildingName=buildingService.queryBuildingName(buildingId);
            //判断实时表是否有记录
            int r1 = nowDataMapper.selectNowDataCount(buildingId,pumpId);//判断now_data表是否存在记录
            if(r1==0){
                data.setPumpId(pumpId); //热泵编号
                String pumpName="热泵"+pumpId;
                data.setPumpName(pumpName); //热泵名称
                data.setBuildingId(buildingId);
                data.setBuildingName(buildingName);
                nowDataMapper.saveNowData(data);
            } else {
                NowDataEntity data1=nowDataMapper.selectNowData(buildingId,pumpId);
                if (data1!=null){
                    if (dataType.equals("tempSet")){         //温度设定
                        data1.setTempSet(strData);
                    } else if (dataType.equals("waterTemp")){ //水箱水温
                        BigDecimal bigDecimal = new BigDecimal(strData);
                        if (bigDecimal.compareTo(new BigDecimal(0)) >= 0
                                && bigDecimal.compareTo(new BigDecimal(100)) <= 0) {
                            data1.setWaterTemp(strData);
                        }
                    }else if (dataType.equals("runState")){  //运行状态
                        data1.setRunState(strData);
                    }else if (dataType.equals("isFault")){   //是否故障
                        data1.setIsFault(strData);
                    }
                    nowDataMapper.updateNowData(data1);
                }
            }
            //判断历史表是否有记录
            int r2 = nowDataMapper.selectHistoryDataCount(strDate,buildingId,pumpId);//判断now_data表是否存在记录
            if (r2==0){
                data.setPumpId(pumpId); //热泵编号
                String pumpName="热泵"+pumpId;
                data.setPumpName(pumpName); //热泵名称
                data.setBuildingId(buildingId);
                data.setBuildingName(buildingName);
                data.setCurDate(strDate);
                nowDataMapper.saveHistoryData(data);
            }else {
                NowDataEntity data2=nowDataMapper.selectHistoryData(strDate,buildingId,pumpId);
                if (data2!=null){
                    if (dataType.equals("tempSet")){         //温度设定
                        data2.setTempSet(strData);
                    }else if (dataType.equals("waterTemp")){ //水箱水温
                        data2.setWaterTemp(strData);
                    }else if (dataType.equals("runState")){  //运行状态
                        data2.setRunState(strData);
                    }else if (dataType.equals("isFault")){   //是否故障
                        data2.setIsFault(strData);
                    }
                    nowDataMapper.updateHistoryData(data2);
                }
            }
        }catch (Exception e){
            e.printStackTrace();
        }
    }

    @Override
    public void updateNowData(NowDataEntity nowDataEntity) {
        nowDataMapper.updateNowData(nowDataEntity);
    }

    @Override
    public List<NowDataEntity> queryNowData(String buildingId) {
        nowDataMapper.updateTempSet(buildingId);
        nowDataMapper.updateLevelSet(buildingId);
        return nowDataMapper.queryNowData(buildingId);
    }

    @Override
    public NowDataEntity queryNowDataByPump(String buildingId,String pumpId) {
        return nowDataMapper.queryNowDataByPump(buildingId,pumpId);
    }

    @Override
    public void saveHistoryData(NowDataEntity nowDataEntity) {
        nowDataMapper.saveHistoryData(nowDataEntity);
    }

    @Override
    public void updateHistoryData(NowDataEntity nowDataEntity) {
        nowDataMapper.updateHistoryData(nowDataEntity);
    }

    @Override
    public List<NowDataEntity> queryHistoryData(String curDate, String buildingId,String pumpId,String tankId, int page, int limit) {
        return nowDataMapper.queryHistoryData(curDate,buildingId,pumpId,tankId,page,limit);
    }

    @Override
    public int getHistoryDataCount(String curDate, String buildingId,String pumpId,String tankId, int page, int limit) {
        return nowDataMapper.getHistoryDataCount(curDate,buildingId,pumpId,tankId,page,limit);
    }

    @Override
    public List<WaterLevelEntity> queryWaterLevel(String curDate, String buildingID, int page, int limit) {
        return nowDataMapper.queryWaterLevel(curDate,buildingID,page,limit);
    }

    @Override
    public int getWaterLevelCount(String curDate, String buildingID) {
        return nowDataMapper.getWaterLevelCount(curDate,buildingID);
    }

    @Override
    public List<WaterLevelModel> queryWaterLevelByTime(String curDate,int page,int limit) {
        return nowDataMapper.queryWaterLevelByTime(curDate,page,limit);
    }

    @Override
    public int waterLevelByTimeCount(String curDate) {
        return nowDataMapper.waterLevelByTimeCount(curDate);
    }

    //从监视表查询热泵温度平均值
    @Override
    public String selectAve(String buildingId) {
//        int pumpCount = nowDataMapper.selectPumpCount(buildingId);
//        Integer temp = null;
//        if (pumpCount > 1) {
//            temp = 40;
//        }
        return nowDataMapper.selectAve(buildingId, null);
    }

    @Override
    public String selectSingleTemp(String pumpId,String buildingId) {
        return nowDataMapper.selectSingleTemp(pumpId,buildingId);
    }

    //从监视表查询热泵运行状态
    @Override
    public String selectStateCount(String buildingId) {
        int count=nowDataMapper.selectStateCount(buildingId);
        String state="";
        if (count==0){
            state="不运行";
        }else{
            state="运行";
        }
        return state;
    }

    @Override
    public String selectState(String buildingId, String pumpId) {
        return nowDataMapper.selectState(buildingId,pumpId);
    }

    @Override
    public void upLevelSet(String buildingId, String levelSet) {
        nowDataMapper.upLevelSet(buildingId,levelSet);
    }

    @Override
    public void upTempSet(String buildingId, String tempSet) {
        nowDataMapper.upTempSet(buildingId,tempSet);
    }

    @Override
    public void upTempSet2(String buildingId, String tempSet, String pumpID) {
        nowDataMapper.upTempSet2(buildingId, tempSet, pumpID);
    }

    @Override
    public List<WaterTempEntity> queryWaterTemp(String buildingID, String curDate,int page,int limit) {
        return nowDataMapper.queryWaterTemp(buildingID,curDate,page,limit);
    }

    @Override
    public int queryWaterTempCount(String buildingID, String curDate) {
        return nowDataMapper.queryWaterTempCount(buildingID,curDate);
    }

    //查询所有楼栋每栋楼的平均温度
    @Override
    public List<WaterTempEntity> queryWaterTemp2(String curDate,int page,int limit) {
        return nowDataMapper.queryWaterTemp2(curDate,page,limit);
    }
    //查询所有楼栋每栋楼的平均温度记录数
    @Override
    public int queryWaterTempCount2(String curDate) {
        return nowDataMapper.queryWaterTempCount2(curDate);
    }

    //调用过程proWaterTemp
    @Override
    public void proWaterTemp(String curDate, String buildingID, String pumpID) {
        lockA.lock();
        try {
            lockB.lock();
            try {
                nowDataMapper.proWaterTemp(curDate,buildingID,pumpID);
            } finally {
                lockB.unlock();
            }
        } finally {
            lockA.unlock();
        }
    }

    @Override
    public List<WaterLevelEntity> queryBuildWaterLevel(String curDate, int page, int limit) {
        return nowDataMapper.queryBuildWaterLevel(curDate, page, limit);
    }

    @Override
    public int buildWaterLevelCount(String curDate) {
        return nowDataMapper.buildWaterLevelCount(curDate);
    }

    @Override
    public void proWaterLevel(String curDate, String buildingID, String deviceAddr) {
        nowDataMapper.proWaterLevel(curDate, buildingID, deviceAddr);
    }

    @Override
    public void updateRunState(String buildingId, String pumpId,String strState, String buildingName) {
        DeviceInstallEntity deviceInstallEntity=deviceInstallMapper.selectDevice(pumpId,"热泵",buildingId);
        NowDataEntity data=new NowDataEntity();
        int r1 = nowDataMapper.selectNowDataCount(buildingId,pumpId);//判断now_data表是否存在记录
        if(r1==0){
            data.setPumpId(pumpId); //通讯地址
            data.setPumpName(deviceInstallEntity.getDeviceName()); //通讯地址
            data.setBuildingId(buildingId);
            data.setBuildingName(buildingName);
            data.setRunState(strState);
            nowDataMapper.saveNowData(data);
        }
        nowDataMapper.updateRunState(buildingId,pumpId,strState);
    }

    @Override
    public void proPumpMinutes(String buildingId, String pumpId, String runState) {
        nowDataMapper.proPumpMinutes(buildingId,pumpId,runState);
    }

    @Override
    public List<PumpMinutesEntity> pumpMinutes(String startDate,String endDate, String buildingId, String pumpId, int page, int limit) {
        return nowDataMapper.pumpMinutes(startDate,endDate,buildingId,pumpId,page,limit);
    }

    @Override
    public int pumpMinutesCount(String startDate,String endDate, String buildingId, String pumpId) {
        return nowDataMapper.pumpMinutesCount(startDate,endDate,buildingId,pumpId);
    }

    @Override
    public List<PumpMinutesEntity> pumpWeekMinutes(String startDate, String endDate, String buildingId, String pumpId, int page, int limit) {
        return nowDataMapper.pumpWeekMinutes(startDate, endDate, buildingId, pumpId, page, limit);
    }

    @Override
    public int pumpWeekMinutesCount(String startDate, String endDate, String buildingId, String pumpId) {
        return nowDataMapper.pumpWeekMinutesCount(startDate, endDate, buildingId, pumpId);
    }

    @Override
    public List<PumpMinutesEntity> pumpMonthMinutes(String startDate, String endDate, String buildingId, String pumpId, int page, int limit) {
        return nowDataMapper.pumpMonthMinutes(startDate, endDate, buildingId, pumpId, page, limit);
    }

    @Override
    public int pumpMonthMinutesCount(String startDate, String endDate, String buildingId, String pumpId) {
        return nowDataMapper.pumpMonthMinutesCount(startDate, endDate, buildingId, pumpId);
    }

    @Override
    public String selectMinPumpId(String buildingId) {
        return nowDataMapper.selectMinPumpId(buildingId);
    }
}
