package com.mh.user.service.impl;

import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import com.mh.common.page.ColumnFilter;
import com.mh.common.page.MybatisPageHelper;
import com.mh.common.page.PageRequest;
import com.mh.common.page.PageResult;
import com.mh.user.entity.DBEntity;
import com.mh.user.mapper.SysRoleMapper;
import com.mh.user.mapper.SysUserMapper;
import com.mh.user.mapper.SysUserRoleMapper;
import com.mh.user.model.SysMenu;
import com.mh.user.model.SysRole;
import com.mh.user.model.SysUser;
import com.mh.user.model.SysUserRole;
import com.mh.user.service.SysMenuService;
import com.mh.user.service.SysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;


@Service
public class SysUserServiceImpl  implements SysUserService {

	/**
	 * 获取用户对应的数据库信息
	 * @param username
	 * @return
	 */
	@Override
	public DBEntity queryDBInfo(String username) {
		return sysUserMapper.queryDBInfo(username);
	}

	@Autowired(required = false)
	private SysUserMapper sysUserMapper;

	@Autowired
	private SysMenuService sysMenuService;

	@Autowired(required = false)
	private SysUserRoleMapper sysUserRoleMapper;

	@Autowired(required = false)
	private SysRoleMapper sysRoleMapper;

	@Transactional
	@Override
	public int save(SysUser record) {
		Long id = null;
		if(record.getId() == null || record.getId() == 0) {
			// 新增用户
			sysUserMapper.insertSelective(record);
			SysUser sysUser=sysUserMapper.findByName(record.getUserName());
			id = sysUser.getId();
		} else {
			// 更新用户信息
			sysUserMapper.updateByPrimaryKeySelective(record);
		}

		// 更新用户角色
		if(id != null && id == 0) {
			return 1;
		}
		if(id != null) {
//			for(SysUserRole sysUserRole:record.getUserRoles()) {
//				sysUserRole.setUserId(id);
//			}

		} else {
			sysUserRoleMapper.deleteByUserId(record.getId());
			id=record.getId();
		}
//		for(SysUserRole sysUserRole:record.getUserRoles()) {
//			sysUserRoleMapper.insertSelective(sysUserRole);
//		}
		SysRole sysRole=sysRoleMapper.findId(record.getRoleNames()); //获取角色编号
		SysUserRole sysUserRole=new SysUserRole();
		sysUserRole.setUserId(id);
		sysUserRole.setRoleId(sysRole.getId());

		sysUserRoleMapper.insertSelective(sysUserRole);
		return 1;
	}

	@Override
	public int delete(SysUser record) {
		return sysUserMapper.deleteByPrimaryKey(record.getId());
	}

	@Override
	public int delete(List<SysUser> records) {
		for(SysUser record:records) {
			delete(record);
		}
		return 1;
	}

	@Override
	public SysUser findById(Long id) {
		return sysUserMapper.selectByPrimaryKey(id);
	}

	@Override
	public SysUser findByName(String name) {
		return sysUserMapper.findByName(name);
	}

	@Override
	public PageResult findPage(PageRequest pageRequest) {
		PageResult pageResult = null;
		String name = getColumnFilterValue(pageRequest, "name");
		String email = getColumnFilterValue(pageRequest, "email");
		if(name != null) {
			if(email != null) {
				pageResult = MybatisPageHelper.findPage(pageRequest, sysUserMapper, "findPageByNameAndEmail", name, email);
			} else {
				pageResult = MybatisPageHelper.findPage(pageRequest, sysUserMapper, "findPageByName", name);
			}
		} else {
			pageResult = MybatisPageHelper.findPage(pageRequest, sysUserMapper);
		}
		// 加载用户角色信息
		findUserRoles(pageResult);
		return pageResult;
	}

	/**
	 * 获取过滤字段的值
	 * @param filterName
	 * @return
	 */
	public String getColumnFilterValue(PageRequest pageRequest, String filterName) {
		String value = null;
		ColumnFilter columnFilter = pageRequest.getColumnFilter(filterName);
		if(columnFilter != null) {
			value = columnFilter.getValue();
		}
		return value;
	}

	/**
	 * 加载用户角色
	 * @param pageResult
	 */
	private void findUserRoles(PageResult pageResult) {
		List<?> content = pageResult.getContent();
		for(Object object:content) {
			SysUser sysUser = (SysUser) object;
			List<SysUserRole> userRoles = findUserRoles(sysUser.getId());
			sysUser.setUserRoles(userRoles);
			sysUser.setRoleNames(getRoleNames(userRoles));
		}
	}

	private String getRoleNames(List<SysUserRole> userRoles) {
		StringBuilder sb = new StringBuilder();
		for(Iterator<SysUserRole> iter = userRoles.iterator(); iter.hasNext();) {
			SysUserRole userRole = iter.next();
			SysRole sysRole = sysRoleMapper.selectByPrimaryKey(userRole.getRoleId());
			if(sysRole == null) {
				continue ;
			}
			sb.append(sysRole.getRemark());
			if(iter.hasNext()) {
				sb.append(", ");
			}
		}
		return sb.toString();
	}

	@Override
	public Set<String> findPermissions(String userName) {
		Set<String> perms = new HashSet<>();
		List<SysMenu> sysMenus = sysMenuService.findByUser(userName);
		for(SysMenu sysMenu:sysMenus) {
			if(sysMenu.getPerms() != null && !"".equals(sysMenu.getPerms())) {
				perms.add(sysMenu.getPerms());
			}
		}
		return perms;
	}

	@Override
	public List<SysUserRole> findUserRoles(Long userId) {
		return sysUserRoleMapper.findUserRoles(userId);
	}

	@Override
	public List<SysUser> queryUsers(String userName, int page, int limit) {
		return sysUserMapper.queryUsers(userName,page,limit);
	}

	@Override
	public int getCount(String userName, int page, int limit) {
		return sysUserMapper.getCount(userName,page,limit);
	}

	@Override
	public void updatePassword(String password,String salt, String id) {
		sysUserMapper.updatePassword(password,salt,id);
	}

	@Override
	public void updateLoginTime(String userName) {
		sysUserMapper.updateLoginTime(userName);
	}
}
