package com.mh.user;

import com.alibaba.fastjson2.JSONObject;
import com.mh.user.constants.Constant;
import com.mh.user.model.SysUser;
import com.mh.user.serialport.SerialPortThread;
import com.mh.user.utils.CacheUtil;
import com.mh.user.utils.ComThreadPoolService;
import io.jsonwebtoken.Claims;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;
import org.checkerframework.checker.units.qual.K;
import sun.security.provider.MD5;

import java.math.BigInteger;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.*;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ThreadPoolExecutor;

/**
 * @author ljf
 * @title ：
 * @description ： 测试jwt认证
 * @updateTime 2020-04-27
 * @throws ：
 */
public class TestJwtUtils {

    public static final String TOKEN_HEADER = "Authorization";
    public static final String TOKEN_PREFIX = "Bearer ";

    public static final String SUBJECT = "congge";

    public static final long EXPIRITION = 1000 * 24 * 60 * 60 * 7;

    public static final String APPSECRET_KEY = "congge_secret";

    private static final String ROLE_CLAIMS = "rol";

    public static String generateJsonWebToken(SysUser user) {

        if (user.getId() == null || user.getUserName() == null) {
            return null;
        }

        Map<String,Object> map = new HashMap<>();
        map.put(ROLE_CLAIMS, "rol");

        String token = Jwts
                .builder()
                .setSubject(SUBJECT)
                .setClaims(map)
                .claim("id", user.getId())
                .claim("name", user.getUserName())
                .setIssuedAt(new Date())
                .setExpiration(new Date(System.currentTimeMillis() + EXPIRITION))
                .signWith(SignatureAlgorithm.HS256, APPSECRET_KEY).compact();
        return token;
    }

    /**
     * 生成token
     * @param username
     * @param role
     * @return
     */
    public static String createToken(String username,String role) {

        Map<String,Object> map = new HashMap<>();
        map.put(ROLE_CLAIMS, role);

        String token = Jwts
                .builder()
                .setSubject(username)
                .setClaims(map)
                .claim("username",username)
                .setIssuedAt(new Date())
                .setExpiration(new Date(System.currentTimeMillis() + EXPIRITION))
                .signWith(SignatureAlgorithm.HS256, APPSECRET_KEY).compact();
        return token;
    }

    public static Claims checkJWT(String token) {
        try {
            final Claims claims = Jwts.parser().setSigningKey(APPSECRET_KEY).parseClaimsJws(token).getBody();
            return claims;
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    /**
     * 获取用户名
     * @param token
     * @return
     */
    public static String getUsername(String token){
        Claims claims = Jwts.parser().setSigningKey(APPSECRET_KEY).parseClaimsJws(token).getBody();
        return claims.get("username").toString();
    }

    /**
     * 获取用户角色
     * @param token
     * @return
     */
    public static String getUserRole(String token){
        Claims claims = Jwts.parser().setSigningKey(APPSECRET_KEY).parseClaimsJws(token).getBody();
        return claims.get("rol").toString();
    }

    /**
     * 是否过期
     * @param token
     * @return
     */
    public static boolean isExpiration(String token){
        Claims claims = Jwts.parser().setSigningKey(APPSECRET_KEY).parseClaimsJws(token).getBody();
        return claims.getExpiration().before(new Date());
    }

//    public static void main(String[] args) {
//        String name = "admin";
//        String token = createToken(name,"admin");
//        System.out.println(token);
//
//        Claims claims = checkJWT(token);
//        System.out.println(claims.get("username"));
//
//        System.out.println(getUsername(token));
//        System.out.println(getUserRole(token));
//        System.out.println(isExpiration(token));
//
//    }

    static ThreadPoolExecutor comThreadPool = ComThreadPoolService.getInstance();

    /**
     * eyJhbGciOiJIUzI1NiJ9.
     * eyJzdWIiOiJjb25nZ2UiLCJpZCI6IjExMDExIiwibmFtZSI6Im51b3dlaXNpa2kiLCJpbWciOiJ3d3cudW9rby5jb20vMS5wbmciLCJpYXQiOjE1NTQ5OTI1NzksImV4cCI6MTU1NTU5NzM3OX0.
     * 6DJ9En-UBcTiMRldZeevJq3e1NxJgOWryUyim4_-tEE
     *
     * @param args
     */
    public static void main(String[] args) {
        // 创建一个按照键的自然顺序排序的TreeMap
        TreeMap<String, Integer> sortedMap = new TreeMap<>();

        // 添加元素到TreeMap
        sortedMap.put("Apple", 1);
        sortedMap.put("Banana", 2);
        sortedMap.put("Cherry", 3);
        sortedMap.put("Date", 4);

        // 输出有序的Map
        for (Map.Entry<String, Integer> entry : sortedMap.entrySet()) {
            System.out.println("Key: " + entry.getKey() + ", Value: " + entry.getValue());
        }
    }

    private static void createBatchSend() throws InterruptedException {
        for (int a = 0; a < 100; a++) {
            if (comThreadPool.isShutdown()) {
                comThreadPool = ComThreadPoolService.getInstance();
            }
            Map<String, Object> dataComMap = new HashMap<>();
            dataComMap.put("1", "1");
            dataComMap.put("2", "1");
            dataComMap.put("3", "1");
            dataComMap.put("4", "1");
            dataComMap.put("5", "1");
            dataComMap.put("6", "1");
            dataComMap.put("7", "1");
            dataComMap.put("8", "1");
            dataComMap.put("9", "1");
            dataComMap.put("10", "1");
            dataComMap.put("11", "1");
            dataComMap.put("12", "1");
            int batchSize = 5; // 定义一个批次大小
            int index = 0;
            for (int k = 0; k < dataComMap.size(); ) {
                System.out.println("第几轮==>" + k);
                CountDownLatch countDownLatch = new CountDownLatch(Math.min(batchSize, dataComMap.size() - k));
                index = k;
                for (int j = 0; j < Math.min(batchSize, dataComMap.size() - k); j++) {
                    if (Constant.WEB_FLAG) {
                        break;
                    }
                    // 获取data_com口
                    String dataCom = dataComMap.keySet().toArray(new String[0])[index];
                    index++;
                    CountDownLatch finalCountDownLatch = countDownLatch;
                    Thread thread = new Thread(() -> {
                        finalCountDownLatch.countDown();
                        System.out.println(Thread.currentThread() + ": " + dataCom);
                    });
                    comThreadPool.execute(thread);
                }
                // 等待执行完成
                countDownLatch.await();
                // 释放资源
                countDownLatch = null;
                // 检查是否需要继续创建新的线程（未达到数据总数量且WEB_FLAG为false）
                if (k + batchSize < dataComMap.size() && !Constant.WEB_FLAG) {
                    k += batchSize;
                } else {
                    break;
                }
            }
            System.out.println("结束==》"+a);
            comThreadPool.shutdown();
        }
    }

}
