package com.mh.user.socket;

import lombok.extern.slf4j.Slf4j;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.nio.ByteBuffer;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Set;

/**
 * @author ljf
 * @title ： NIO 非阻塞式编程
 * @description ：
 * @updateTime 2020-04-20
 * @throws ：
 */
@Slf4j
public class NIOServer {

    /**
     * 已经建立连接的集合
     */
    private static ArrayList<SocketChannel> channels = new ArrayList<>();

    public static void main(String[] args) throws IOException {
        log.info("test");
        NIOServer nioServer = new NIOServer();
        // 缓冲测试
//        nioServer.BufferTest();
        // 通道测试
//        nioServer.ChannelTest();
        // Selector单线程操作
        nioServer.SelectorSingleTest();
    }

    // 缓冲区使用
    public void BufferTest(){
//        // 构建一个堆内字节缓冲区
//        ByteBuffer byteBuffer = ByteBuffer.allocate(4);
        // 构建一个堆外字节缓冲区
        ByteBuffer byteBuffer = ByteBuffer.allocateDirect(4);
        // 默认写入模式，查看三个重要的指标
        log.info(String.format("初始化：capacity容量：%s, position位置：%s, limit限制：%s", byteBuffer.capacity(),
                byteBuffer.position(), byteBuffer.limit()));
        // 写入字节
        byteBuffer.put((byte) 1);
        byteBuffer.put((byte) 2);
        byteBuffer.put((byte) 3);
        // 查看占用多少
        log.info(String.format("初始化：capacity容量：%s, position位置：%s, limit限制：%s", byteBuffer.capacity(),
                byteBuffer.position(), byteBuffer.limit()));

        // 转换为读取方式（如果不调用buffer.flip方式，读取的顺序不对）
        byteBuffer.flip();
        // 读取
        byte a = byteBuffer.get();
        log.info(""+a);
        byte b = byteBuffer.get();
        log.info(""+b);
        // 查看占用多少
        log.info(String.format("读取后的结果：capacity容量：%s, position位置：%s, limit限制：%s", byteBuffer.capacity(),
                byteBuffer.position(), byteBuffer.limit()));
        // 继续写入3字节，此时读模式下，limit=3，position=2.继续写入只能覆盖写入一条数据
        // clear()方法清除整个缓冲区。compact()方法仅清除已阅读的数据。转为写入模式
        byteBuffer.compact(); // buffer内部还残留1个数据,还可以写3个数据
        byteBuffer.put((byte) 3);
        byteBuffer.put((byte) 4);
        byteBuffer.put((byte) 5);
        // 查看占用多少
        log.info(String.format("最终的结果：capacity容量：%s, position位置：%s, limit限制：%s", byteBuffer.capacity(),
                byteBuffer.position(), byteBuffer.limit()));
    }

    // Channel通道测试, 基于非阻塞式
    public void ChannelTest() throws IOException {
        // 创建网络服务端
        ServerSocketChannel serverSocketChannel = ServerSocketChannel.open();
        // 设置成非阻塞式
        serverSocketChannel.configureBlocking(false);
        // 绑定端口号
        serverSocketChannel.socket().bind(new InetSocketAddress(9093));
        log.info("启动成功");
        while(true){
            // 获取新tcp连接通道
            SocketChannel socketChannel = serverSocketChannel.accept();
//            // tcp 请求、响应
//            if (socketChannel != null){
//                log.info("收到新连接："+serverSocketChannel.getLocalAddress());
//                // 设置非阻塞式
//                serverSocketChannel.configureBlocking(false);
//                try {
//                    ByteBuffer requestBuffer = ByteBuffer.allocate(1024);
//                    while (socketChannel.isOpen() && socketChannel.read(requestBuffer) != -1) {
//                        // 长连接情况下,需要手动判断数据有没有读取结束 (此处做一个简单的判断: 超过0字节就认为请求结束了)
//                        if (requestBuffer.position() > 0)
//                            break;
//                    }
//                    if (requestBuffer.position() == 0)
//                        continue; // 如果没数据了, 则不继续后面的处理
//                    requestBuffer.flip();
//                    byte[] content = new byte[requestBuffer.limit()];
//                    requestBuffer.get(content);
//                    log.info(new String(content));
//                    log.info("收到数据,来自：" + socketChannel.getRemoteAddress());
//
//                    // 响应结果 200
//                    String response = "HTTP/1.1 200 OK\r\n" + "Content-Length: 11\r\n\r\n" + "Hello World";
//                    ByteBuffer buffer = ByteBuffer.wrap(response.getBytes());
//                    while (buffer.hasRemaining()) {
//                        socketChannel.write(buffer);// 非阻塞
//                    }
//                } catch (IOException e) {
//                    e.printStackTrace();
//                }
//            }
            // tcp请求 读取/响应
            if (socketChannel != null) {
                log.info("收到新连接 : " + socketChannel.getRemoteAddress());
                socketChannel.configureBlocking(false); // 默认是阻塞的,一定要设置为非阻塞
                channels.add(socketChannel);
            } else {
                // 没有新连接的情况下,就去处理现有连接的数据,处理完的就删除掉
                Iterator<SocketChannel> iterator = channels.iterator();
                while (iterator.hasNext()) {
                    SocketChannel ch = iterator.next();
                    try {
                        ByteBuffer requestBuffer = ByteBuffer.allocate(1024);
                        // 如果通道里没有数据，就没必要再继续下面的循环读取了
                        if (ch.read(requestBuffer) == 0) {
                            // 等于0,代表这个通道没有数据需要处理,那就待会再处理
                            continue;
                        }
                        while (ch.isOpen() && ch.read(requestBuffer) != -1) {
                            // 长连接情况下,需要手动判断数据有没有读取结束 (此处做一个简单的判断: 超过0字节就认为请求结束了)
                            if (requestBuffer.position() > 0)
                                break;
                        }
                        if (requestBuffer.position() == 0)
                            continue; // 如果没数据了, 则不继续后面的处理
                        requestBuffer.flip();
                        byte[] content = new byte[requestBuffer.limit()];
                        requestBuffer.get(content);
                        log.info(new String(content));
                        log.info("收到数据,来自：" + ch.getRemoteAddress());

                        // 响应结果 200
                        String response = "HTTP/1.1 200 OK\r\n" + "Content-Length: 11\r\n\r\n" + "Hello World";
                        ByteBuffer buffer = ByteBuffer.wrap(response.getBytes());
                        while (buffer.hasRemaining()) {
                            ch.write(buffer);
                        }
                        iterator.remove();
                    } catch (IOException e) {
                        e.printStackTrace();
                        iterator.remove();
                    }
                }
            }
        }
    }

    /**
     *  Selector多路复用器（单线程方式） 结合Selector实现的非阻塞服务端(放弃对channel的轮询,借助消息通知机制)
      */
    public void SelectorSingleTest() throws IOException {
        // 1、创建网络服务端ServerSocketChannel
        ServerSocketChannel serverSocketChannel = ServerSocketChannel.open();
        // 设置为非阻塞模式
        serverSocketChannel.configureBlocking(false);

        // 2、创建一个Selector选择器，并将channel注册上去
        Selector selector = Selector.open();
        SelectionKey selectionKey = serverSocketChannel.register(selector,0,serverSocketChannel);       // 注册
        selectionKey.interestOps(SelectionKey.OP_ACCEPT);  // 对serverSocketChannel上面的accept事件感兴趣(serverSocketChannel只能支持accept操作)

        // 3、绑定端口
        serverSocketChannel.socket().bind(new InetSocketAddress(9093));

        log.info("启动Selector程序成功");

        while(true){
            // 不使用轮询通道的方式，使用.select方式阻塞效果，知道有通知才会有返回
            selector.select();
            // 获取事件
            Set<SelectionKey> selectionKeys = selector.selectedKeys();
            // 遍历查询结果
            Iterator<SelectionKey> iterator = selectionKeys.iterator();
            while (iterator.hasNext()){
                // 被封装的查询结果
                SelectionKey key = iterator.next();
                iterator.remove();
                // 关注Read和Accept事件
                if (key.isAcceptable()){
                    ServerSocketChannel serverSocketChannel1 = (ServerSocketChannel) key.attachment();
                    // 将拿到的客户端连接通信，注册到selector上面
                    SocketChannel clientSocketChannel = serverSocketChannel1.accept();      // mainReactor轮询accept
                    clientSocketChannel.configureBlocking(false);                           // 设置非阻塞
                    clientSocketChannel.register(selector,SelectionKey.OP_READ,clientSocketChannel);
                    log.info("收到新连接："+clientSocketChannel.getRemoteAddress());
                }
                // 读取数据
                if (key.isReadable()){
                    SocketChannel socketChannel = (SocketChannel) key.attachment();
                    try{
                        ByteBuffer requestBuffer = ByteBuffer.allocate(1024);
                        while (socketChannel.isOpen() && socketChannel.read(requestBuffer) != -1){
                            // 长时间连接情况下，需要手动进行数据判断，有没有读取结束，超过0字节就是请求结束
                            if (requestBuffer.position() > 0){
                                break;
                            }
                            if (requestBuffer.position() == 0)
                                continue;
                            requestBuffer.flip();
                            byte[] content = new byte[requestBuffer.limit()];
                            requestBuffer.get(content);
                            log.info(new String(content));
                            log.info("收到数据，来自："+socketChannel.getRemoteAddress());
                            // 进行业务操作

                            //进行相应
                            String response = "HTTP/1.1 200 OK\r\n" +
                                    "Content-Length: 11\r\n\r\n" +
                                    "Hello World";
                            ByteBuffer buffer = ByteBuffer.wrap(response.getBytes());
                            while (buffer.hasRemaining()) {
                                socketChannel.write(buffer);
                            }
                        }
                    }catch (IOException e){
                        key.cancel();//取消消息订阅
                    }
                }
            }
            selector.selectNow();
        }
        // 问题: 此处一个selector监听所有事件,一个线程处理所有请求事件. 会成为瓶颈! 要有多线程的运用
    }

}

